using CadEditor;
using System.Collections.Generic;
using System.IO;
using System;
using System.Windows.Forms;
//css_include Settings_CapcomBase.cs;
public class Data:CapcomBase
{
  public override GameType getGameType()  { return GameType.DT2; }
  
  public OffsetRec getPalOffset()       { return new OffsetRec(0x3E2F, 12   , 16);     }
  public OffsetRec getVideoOffset()     { return new OffsetRec(0x4D10 , 5   , 0xD00);  }
  public OffsetRec getVideoObjOffset()  { return new OffsetRec(0x4D10 , 5   , 0xD00);  }
  public OffsetRec getBigBlocksOffset() { return new OffsetRec(0x7310 , 3   , 0x4000); }
  public OffsetRec getBlocksOffset()    { return new OffsetRec(0x1008A , 5  , 0x440);  }
  public OffsetRec getScreensOffset()   { return new OffsetRec(0x11d5a, 300 , 0x40);   }
  public IList<LevelRec> getLevelRecs() { return levelRecsDt2; }
  public string[] getBlockTypeNames()   { return objTypesDt2;  }
  
  public override GetVideoPageAddrFunc getVideoPageAddrFunc() { return getDuckTalesVideoAddress; }
  public override GetVideoChunkFunc    getVideoChunkFunc()    { return getDuckTalesVideoChunk;   }
  public override SetVideoChunkFunc    setVideoChunkFunc()    { return null; }
  public override GetBigBlocksFunc     getBigBlocksFunc()     { return getBigBlocksDt2;}
  public override SetBigBlocksFunc     setBigBlocksFunc()     { return setBigBlocksDt2;}
  
  public IList<LevelRec> levelRecsDt2 = new List<LevelRec>() 
  {
    new LevelRec(0x19488, 0xFF, 8, 6, 0x11C3A),
    new LevelRec(0x195A7, 0xFF, 8, 6, 0x11C6A),
    new LevelRec(0x196E7, 0xFF, 8, 6, 0x11C9A),
    new LevelRec(0x19830, 0xFF, 8, 6, 0x11CCA),
    new LevelRec(0x19970, 0xFF, 8, 6, 0x11CFA),
    new LevelRec(0x19A87, 0xFF, 8, 6, 0x11D2A),
    new LevelRec(0x19B9E, 0xFF, 8, 6, 0x11C6A),
  };
  
  string[] objTypesDt2 = new[] {
    "no","no","no","no","no","no","no","no",
    "no","no","no","no","no","no","no","no"
  };
  
  //--------------------------------------------------------------------------------------------
  //duck tales specific
  public int getDuckTalesVideoAddress(int id)
  {
    return -1;
  }
  
  public byte[] getDuckTalesVideoChunk(int videoPageId)
  {
    try
    {
        using (FileStream f = File.OpenRead("videoBack_DT2.bin"))
        {
            byte[] videodata = new byte[0x5000];
            f.Read(videodata, 0, 0x5000);
            byte[] ans = new byte[0x1000];
            int offset = (videoPageId - 0x90)*0x1000;
            for (int i = 0; i < ans.Length; i++)
                ans[i] = videodata[offset + i];
            return ans;
        }
    }
    catch (Exception ex)
    {
        MessageBox.Show(ex.Message);
    }
    return null;
  }
  
  public int getBigBlocksCountForLevel(int levelNo)
  {
    int[] bigBlocksCount = {207,195,169,176,209,209};
    return bigBlocksCount[levelNo];
  }
  
  public int[] getBigBlocksPtrsForLevel(int levelNo)
  {
    int[][] levelPointers = { 
     new int[]{0x10D4A, 0x10E19, 0x10EE8, 0x10FB7},  //ptrs for level 1
     new int[]{0x11086, 0x11149, 0x1120C, 0x112CF},  //ptrs for level 2
     new int[]{0x11392, 0x1143B, 0x114E4, 0x1158D},  //ptrs for level 3
     new int[]{0x11636, 0x116E6, 0x11796, 0x11846},  //ptrs for level 4
     new int[]{0x118F6, 0x119C7, 0x11A98, 0x11B69},  //ptrs for level 5
     new int[]{0x118F6, 0x119C7, 0x11A98, 0x11B69}   //ptrs for level 6 (same as 5)
    };
    return levelPointers[levelNo];
  }
  
  public byte[] getBigBlocksDt2(int bigTileIndex)
  {
    int[] addrPointers = getBigBlocksPtrsForLevel(bigTileIndex);
    int blocksCount = getBigBlocksCountForLevel(bigTileIndex);
    byte[] bigBlockIndexes = new byte[getBigBlocksCount()*4];
    byte[] tempIndexes = Utils.readDataFromUnalignedArrays(Globals.romdata, addrPointers[0], addrPointers[1], addrPointers[2], addrPointers[3], blocksCount);
    Array.Copy(tempIndexes, bigBlockIndexes, blocksCount*4);    
    return bigBlockIndexes;
  }
  
  public void setBigBlocksDt2(int bigTileIndex, byte[] data)
  {
    int[] addrPointers = getBigBlocksPtrsForLevel(bigTileIndex);
    int blocksCount = getBigBlocksCountForLevel(bigTileIndex);
    Utils.writeDataToUnalignedArrays(data, Globals.romdata, addrPointers[0], addrPointers[1], addrPointers[2], addrPointers[3], blocksCount); 
  }
  //--------------------------------------------------------------------------------------------
}